@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#coding: utf8
+# coding: utf-8
 """
 FabIO module
 
@@ -8,12 +8,13 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "14/01/2014"
+__date__ = "21/07/2015"
 __status__ = "stable"
 
-version = "0.1.4"
+
 import logging
 logging.basicConfig()
+from ._version import version, version_info, hexversion
 from . import fabioimage
 from . import openimage
 from .fabioutils import COMPRESSORS, jump_filename, FilenameObject, \
@@ -25,3 +26,19 @@ filename_object = FilenameObject
 
 from .openimage import openimage as open
 from .openimage import openheader as openheader
+
+
+
+def tests():
+    """
+    Run the FabIO test suite. 
+    
+    If the test-images are not already installed (via the debian package for example),
+    they need to be downloaded from sourceforge.net, which make take a while.
+    Ensure your network connection is operational and your proxy settings are correct,
+    for example:
+    
+    export http_proxy=http://proxy.site.com:3128 
+    """
+    from . import test
+    test.run_tests()
