@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 """
 
 License: GPLv2+
@@ -19,12 +19,12 @@ Authors:
 
 """
 # get ready for python3
-from __future__ import with_statement, print_function
-import os, logging, types
+from __future__ import with_statement, print_function, absolute_import, division
+import os, logging
 logger = logging.getLogger("edfimage")
 import numpy
 from .fabioimage import fabioimage
-from .fabioutils import isAscii, toAscii, nice_int
+from .fabioutils import isAscii, toAscii, nice_int, six
 from .compression import decBzip2, decGzip, decZlib
 
 
@@ -48,8 +48,8 @@ DATA_TYPES = {  "SignedByte"    :  numpy.int8,
                 "Unsigned64"    :  numpy.uint64,
                 "FloatValue"    :  numpy.float32,
                 "FLOATVALUE"    :  numpy.float32,
-                "FLOAT"         :  numpy.float32, # fit2d
-                "Float"         :  numpy.float32, # fit2d
+                "FLOAT"         :  numpy.float32,  # fit2d
+                "Float"         :  numpy.float32,  # fit2d
                 "FloatIEEE32"   :  numpy.float32,
                 "Float32"       :  numpy.float32,
                 "Double"        :  numpy.float64,
@@ -57,13 +57,11 @@ DATA_TYPES = {  "SignedByte"    :  numpy.int8,
                 "FloatIEEE64"   :  numpy.float64,
                 "DoubleIEEE64"  :  numpy.float64}
 try:
-    DATA_TYPES["FloatIEEE128" ] =  numpy.float128
-    DATA_TYPES["DoubleIEEE128" ] =  numpy.float128
-    DATA_TYPES["QuadrupleValue" ] =  numpy.float128
+    DATA_TYPES["FloatIEEE128"] = DATA_TYPES["DoubleIEEE128"] = DATA_TYPES["QuadrupleValue"] = numpy.float128
 
 except AttributeError:
     # not in your numpy
-    pass
+    logger.debug("No support for float128 in your code")
 
 NUMPY_EDF_DTYPE = {"int8"       :"SignedByte",
                    "int16"      :"SignedShort",
@@ -84,7 +82,7 @@ MINIMUM_KEYS = ['HEADERID',
                 'DATATYPE',
                 'DIM_1',
                 'DIM_2',
-                'SIZE'] # Size is thought to be essential for writing at least
+                'SIZE']  # Size is thought to be essential for writing at least
 
 DEFAULT_VALUES = {
                   # I do not define default values as they will be calculated at write time
@@ -102,12 +100,12 @@ class Frame(object):
             self.header = dict(header)
 
         if header_keys is None:
-            self.header_keys = self.header.keys()
+            self.header_keys = list(self.header.keys())
         else:
             self.header_keys = header_keys[:]
             for key in header_keys:
                 if key not in self.header:
-                    logger.warning("Header key %s, in header_keys is not in header dictionary, poping !!!" % key)
+                    logger.warning("Header key %s, in header_keys is not in header dictionary, popping !!!" % key)
                     self.header_keys.remove(key)
 
         self.capsHeader = {}
@@ -120,7 +118,7 @@ class Frame(object):
         self.dims = []
         self.dim1 = 0
         self.dim2 = 0
-        self.start = None # Position of start of raw data in file
+        self.start = None  # Position of start of raw data in file
         self.size = None  # size of raw data in file
         self.file = None  # opened file object with locking capabilities !!!
         self.bpp = None
@@ -134,11 +132,11 @@ class Frame(object):
         """
         Parse the header in some EDF format from an already open file
 
-        @param block: string representing the header block
+        @param block: string representing the header block.
         @type block: string, should be full ascii
         @return: size of the binary blob
         """
-        #reset values ...
+        # reset values ...
         self.header = {}
         self.capsHeader = {}
         self.header_keys = []
@@ -150,8 +148,8 @@ class Frame(object):
             if '=' in line:
                 key, val = line.split('=' , 1)
                 # Why would someone put null bytes in a header?
-                key = key.replace("\x00"," ").strip()
-                self.header[key] = val.replace("\x00"," ").strip()
+                key = key.replace("\x00", " ").strip()
+                self.header[key] = val.replace("\x00", " ").strip()
                 self.capsHeader[key.upper()] = key
                 self.header_keys.append(key)
 
@@ -273,8 +271,8 @@ class Frame(object):
                     uncompressed_size *= i
                 if "OFFSET" in compression :
                     try:
-                        import byte_offset#IGNORE:F0401
-                    except ImportError, error:
+                        import byte_offset  # IGNORE:F0401
+                    except ImportError as error:
                         logger.error("Unimplemented compression scheme:  %s (%s)" % (compression, error))
                     else:
                         myData = byte_offset.analyseCython(fileData, size=uncompressed_size)
@@ -306,13 +304,13 @@ class Frame(object):
             elif expected < len(rawData):
                 logger.info("Data stream contains trailing junk : %s > expected %s bytes" % (obtained, expected))
                 rawData = rawData[:expected]
+            data = numpy.fromstring(rawData, self._bytecode).reshape(tuple(dims))
             if self.swap_needed():
-                data = numpy.fromstring(rawData, self._bytecode).byteswap().reshape(tuple(dims))
-            else:
-                data = numpy.fromstring(rawData, self._bytecode).reshape(tuple(dims))
+                data.byteswap(True)
             self._data = data
             self._bytecode = data.dtype.type
         return data
+
     def setData(self, npa=None):
         """Setter for data in edf frame"""
         self._data = npa
@@ -330,10 +328,10 @@ class Frame(object):
         """
         @param force_type: type of the dataset to be enforced like "float64" or "uint16"
         @type force_type: string or numpy.dtype
-        @param fit2dMode: enforce compatibility with fit2d and starts countimg number of images at 1
+        @param fit2dMode: enforce compatibility with fit2d and starts counting number of images at 1
         @type fit2dMode: boolean
-        @return: ascii header block
-        @rtype: python string with the concatenation of the ascii header and the binary data block
+        @return: ascii header block + binary data block
+        @rtype: python bytes with the concatenation of the ascii header and the binary data block
         """
         if force_type is not None:
             data = self.data.astype(force_type)
@@ -363,7 +361,7 @@ class Frame(object):
                 header_keys.remove(capsHeader[KEY])
         if "EDF_DATABLOCKID" in capsHeader:
             header_keys.remove(capsHeader["EDF_DATABLOCKID"])
-            #but do not remove the value from dict, instead reset the key ...
+            # but do not remove the value from dict, instead reset the key ...
             if capsHeader["EDF_DATABLOCKID"] != "EDF_DataBlockID":
                 header["EDF_DataBlockID"] = header.pop(capsHeader["EDF_DATABLOCKID"])
                 capsHeader["EDF_DATABLOCKID"] = "EDF_DataBlockID"
@@ -397,13 +395,13 @@ class Frame(object):
         header_keys.insert(0, "EDF_HeaderSize")
         header["EDF_HeaderSize"] = "%5s" % (approxHeaderSize)
         header_keys.insert(0, "EDF_BinarySize")
-        header["EDF_BinarySize"] = len(data.tostring())
+        header["EDF_BinarySize"] = data.nbytes
         header_keys.insert(0, "EDF_DataBlockID")
         if not "EDF_DataBlockID" in header:
             header["EDF_DataBlockID"] = "%i.Image.Psd" % (self.iFrame + fit2dMode)
-        preciseSize = 4 #2 before {\n 2 after }\n
+        preciseSize = 4  # 2 before {\n 2 after }\n
         for key in header_keys:
-            #Escape keys or values that are no ascii
+            # Escape keys or values that are no ascii
             strKey = str(key)
             if not isAscii(strKey, listExcluded=["}", "{"]):
                 logger.warning("Non ascii key %s, skipping" % strKey)
@@ -431,7 +429,7 @@ class Frame(object):
         else:
             headerSize = approxHeaderSize
         listHeader.append(" "*(headerSize - preciseSize) + "}\n")
-        return "".join(listHeader) + data.tostring()
+        return ("".join(listHeader)).encode("ASCII") + data.tostring()
 
 
 
@@ -443,7 +441,7 @@ class edfimage(fabioimage):
         self.filesize = None
         try:
             dim = len(data.shape)
-        except Exception, error: #IGNORE:W0703
+        except Exception as error:  # IGNORE:W0703
             logger.debug("Data don't look like a numpy array (%s), resetting all!!" % error)
             data = None
             dim = 0
@@ -473,7 +471,7 @@ class edfimage(fabioimage):
         """
         Empty for fabioimage but may be populated by others classes
         """
-        if type(header) != types.DictionaryType:
+        if not isinstance(header, dict):
             return {}
         new = {}
         for key, value in header.items():
@@ -493,41 +491,40 @@ class edfimage(fabioimage):
         if len(block) < BLOCKSIZE:
             logger.debug("Under-short header: only %i bytes in %s" % (len(block), infile.name))
             return
-        if (block.find("{") < 0) :
+        if (block.find(b"{") < 0) :
             # This does not look like an edf file
             logger.warning("no opening {. Corrupt header of EDF file %s" % infile.name)
             return
-        if "EDF_HeaderSize" in block:
-            start = block.index("EDF_HeaderSize")
-            chunk = block[start:].split("=")[1].strip()
+        if b"EDF_HeaderSize" in block:
+            start = block.index(b"EDF_HeaderSize")
+            chunk = block[start:].split(b"=")[1].strip()
             try:
-                new_max_header_size = int(chunk.split(";")[0].strip())
+                new_max_header_size = int(chunk.split(b";")[0].strip())
             except Exception:
                 logger.warning("Unable to read header size in %s" % chunk)
             else:
                 if new_max_header_size > MAX_HEADER_SIZE:
                     logger.info("Redefining MAX_HEADER_SIZE to %s" % new_max_header_size)
                     MAX_HEADER_SIZE = new_max_header_size
-        while '}' not in block:
+        while b'}' not in block:
             block = block + infile.read(BLOCKSIZE)
             if len(block) > MAX_HEADER_SIZE:
                 logger.warning("Runaway header in EDF file MAX_HEADER_SIZE: %s \n%s" % (MAX_HEADER_SIZE, block))
                 return
-        start = block.find("{") + 1
-        end = block.find("}")
+        start = block.find(b"{") + 1
+        end = block.find(b"}")
 
         # Now it is essential to go to the start of the binary part
-        if block[end: end + 3] == "}\r\n":
+        if block[end: end + 3] == b"}\r\n":
             offset = end + 3 - len(block)
-        elif block[end: end + 2] == "}\n":
+        elif block[end: end + 2] == b"}\n":
             offset = end + 2 - len(block)
         else:
             logger.error("Unable to locate start of the binary section")
             offset = None
         if offset is not None:
             infile.seek(offset, os.SEEK_CUR)
-        return block[start:end]
-
+        return block[start:end].decode("ASCII")
 
     def _readheader(self, infile):
         """
@@ -537,6 +534,15 @@ class edfimage(fabioimage):
         """
         self.__frames = []
         bContinue = True
+        attrs = dir(infile)
+        if "measure_size" in attrs:
+            # Handle bug #18 (https://github.com/kif/fabio/issues/18)
+            stream_size = infile.measure_size()
+        elif "size" in attrs:
+            stream_size = infile.size
+        elif "len" in attrs:
+            stream_size = infile.len
+
         while bContinue:
             block = self._readHeaderBlock(infile)
             if block is None:
@@ -550,14 +556,14 @@ class edfimage(fabioimage):
             self.__frames += [frame]
             try:
                 infile.seek(size, os.SEEK_CUR)
-            except Exception, error:
+            except Exception as error:
                 logger.warning("infile is %s" % infile)
                 logger.warning("Position is %s" % infile.tell())
                 logger.warning("size is %s" % size)
                 logger.error("It seams this error occurs under windows when reading a (large-) file over network: %s ", error)
                 raise Exception(error)
 
-            if  frame.start + size > infile.size:
+            if  frame.start + size > stream_size:
                 logger.warning("Non complete datablock: got %s, expected %s" % (infile.size - frame.start, size))
                 bContinue = False
                 break
@@ -596,16 +602,17 @@ class edfimage(fabioimage):
     def swap_needed(self):
         """
         Decide if we need to byteswap
+
+        @return True if needed, False else and None if not understood
         """
+        if self.bpp == 1 :
+            return False
         if ('Low'  in self.header[self.capsHeader['BYTEORDER']] and numpy.little_endian) or \
            ('High' in self.header[self.capsHeader['BYTEORDER']] and not numpy.little_endian):
             return False
         if ('High'  in self.header[self.capsHeader['BYTEORDER']] and numpy.little_endian) or \
            ('Low' in self.header[self.capsHeader['BYTEORDER']] and not numpy.little_endian):
-            if self.bpp in [2, 4, 8]:
-                return True
-            else:
-                return False
+            return True
 
     def unpack(self):
         """
@@ -622,7 +629,7 @@ class edfimage(fabioimage):
         if self.nframes == 1:
             logger.debug("Single frame EDF; having fabioimage default behavour: %s" % num)
             newImage = fabioimage.getframe(self, num)
-        elif num in xrange(self.nframes):
+        elif num in range(self.nframes):
             logger.debug("Multi frame EDF; having edfimage specific behavour: %s/%s" % (num, self.nframes))
             newImage = edfimage(frames=self.__frames)
             newImage.currentframe = num
@@ -655,7 +662,6 @@ class edfimage(fabioimage):
             newImage = self.getframe(newFrameId)
         return newImage
 
-
     def write(self, fname, force_type=None, fit2dMode=False):
         """
         Try to write a file
@@ -666,13 +672,15 @@ class edfimage(fabioimage):
         @return: None
 
         """
-
+        # correct for bug #27: read all data before opening the file in write mode
+        if fname == self.filename:
+            [(frame.header, frame.data) for frame in self.__frames]
+            # this is thrown away
         with self._open(fname, mode="wb") as outfile:
             for i, frame in enumerate(self.__frames):
                 frame.iFrame = i
                 outfile.write(frame.getEdfBlock(force_type=force_type, fit2dMode=fit2dMode))
 
-
     def appendFrame(self, frame=None, data=None, header=None):
         """
         Method used add a frame to an EDF file
@@ -685,7 +693,6 @@ class edfimage(fabioimage):
         else:
             self.__frames.append(Frame(data, header))
 
-
     def deleteFrame(self, frameNb=None):
         """
         Method used to remove a frame from an EDF image. by default the last one is removed.
@@ -701,8 +708,8 @@ class edfimage(fabioimage):
     def fastReadData(self, filename=None):
         """
         This is a special method that will read and return the data from another file ...
-        The aim is performances, ... but only supports uncompressed files. 
-         
+        The aim is performances, ... but only supports uncompressed files.
+
         @return: data from another file using positions from current edfimage
         """
         if (filename is None) or not os.path.isfile(filename):
@@ -715,15 +722,17 @@ class edfimage(fabioimage):
         try:
             data = numpy.fromstring(raw, dtype=self.bytecode)
             data.shape = self.data.shape
-        except Exception, err :
-            logger.error("unable to convert file content to numpy array: %s", err)
+        except Exception as error:
+            logger.error("unable to convert file content to numpy array: %s", error)
+        if self.swap_needed():
+            data.byteswap(True)
         return data
 
     def fastReadROI(self, filename, coords=None):
         """
         Method reading Region of Interest of another file  based on metadata available in current edfimage.
         The aim is performances, ... but only supports uncompressed files.
-        
+
         @return: ROI-data from another file using positions from current edfimage
         @rtype: numpy 2darray
         """
@@ -753,11 +762,12 @@ class edfimage(fabioimage):
         try:
             data = numpy.fromstring(raw, dtype=self.bytecode)
             data.shape = -1, d1
-        except Exception, err :
-            logger.error("unable to convert file content to numpy array: %s", err)
+        except Exception as error:
+            logger.error("unable to convert file content to numpy array: %s", error)
+        if self.swap_needed():
+            data.byteswap(True)
         return data[slice2]
 
-
 ################################################################################
 # Properties definition for header, data, header_keys and capsHeader
 ################################################################################
