@@ -13,8 +13,8 @@ Authors: Henning O. Sorensen & Erik Knudsen
 
 """
 # get ready for python3
-from __future__ import with_statement, print_function
-import os, types, logging, sys, tempfile
+from __future__ import with_statement, print_function, absolute_import, division
+import os, logging, sys, tempfile, threading
 logger = logging.getLogger("fabioimage")
 import numpy
 try:
@@ -22,7 +22,7 @@ try:
 except ImportError:
     logger.warning("PIL is not installed ... trying to do without")
     Image = None
-import fabioutils, converters
+from . import fabioutils, converters
 
 
 class fabioimage(object):
@@ -39,7 +39,7 @@ class fabioimage(object):
         Set up initial values
         """
         self._classname = None
-        if type(data) in types.StringTypes:
+        if type(data) in fabioutils.StringTypes:
             raise Exception("fabioimage.__init__ bad argument - " + \
                             "data should be numpy array")
         self.data = self.checkData(data)
@@ -48,13 +48,13 @@ class fabioimage(object):
             self.header = {}
         else:
             self.header = self.checkHeader(header)
-        self.header_keys = self.header.keys() # holds key ordering
+        self.header_keys = list(self.header.keys())  # holds key ordering
         if self.data is not None:
             self.dim2, self.dim1 = self.data.shape
         else:
             self.dim1 = self.dim2 = 0
-        self.bytecode = None     # numpy typecode
-        self.bpp = 2             # bytes per pixel
+        self.bytecode = None  # numpy typecode
+        self.bpp = 2  # bytes per pixel
         # cache for image statistics
         self.mean = self.maxval = self.stddev = self.minval = None
         # Cache roi
@@ -98,28 +98,25 @@ class fabioimage(object):
         """ returns the file numbered 'num' in the series as a fabioimage """
         if self.nframes == 1:
             # single image per file
-            import openimage
-            return openimage.openimage(
-                fabioutils.jump_filename(self.filename, num))
+            from .openimage import openimage
+            return openimage(fabioutils.jump_filename(self.filename, num))
         raise Exception("getframe out of range")
 
     def previous(self):
         """ returns the previous file in the series as a fabioimage """
-        import openimage
-        return openimage.openimage(
-            fabioutils.previous_filename(self.filename))
+        from .openimage import openimage
+        return openimage(fabioutils.previous_filename(self.filename))
 
     def next(self):
         """ returns the next file in the series as a fabioimage """
-        import openimage
-        return openimage.openimage(
+        from .openimage import openimage
+        return openimage(
             fabioutils.next_filename(self.filename))
 
     def toPIL16(self, filename=None):
         """
         Convert to Python Imaging Library 16 bit greyscale image
 
-        FIXME - this should be handled by the libraries now
         """
         if not Image:
             raise RuntimeError("PIL is not installed !!! ")
@@ -131,34 +128,19 @@ class fabioimage(object):
         size = self.data.shape[:2][::-1]
         typmap = {
             'float32' : "F"     ,
-            'int32'   : "F;32S" ,
-            'uint32'  : "F;32"  ,
-            'int16'   : "F;16S" ,
-            'uint16'  : "F;16"  ,
+            'int32'   : "F;32NS" ,
+            'uint32'  : "F;32N"  ,
+            'int16'   : "F;16NS" ,
+            'uint16'  : "F;16N"  ,
             'int8'    : "F;8S"  ,
             'uint8'   : "F;8"  }
-        if typmap.has_key(self.data.dtype.name):
+        if self.data.dtype.name in typmap:
             mode2 = typmap[ self.data.dtype.name ]
             mode1 = mode2[0]
         else:
-            raise Exception("Unknown numpy type " + str(self.data.dtype.type))
-        #
-        # hack for byteswapping for PIL in MacOS
-        testval = numpy.array((1, 0), numpy.uint8).view(numpy.uint16)[0]
-        if  testval == 1:
-            dats = self.data.tostring()
-        elif testval == 256:
-            dats = self.data.byteswap().tostring()
-        else:
-            raise Exception("Endian unknown in fabioimage.toPIL16")
-
-        self.pilimage = Image.frombuffer(mode1,
-                                         size,
-                                         dats,
-                                         "raw",
-                                         mode2,
-                                         0,
-                                         1)
+            raise RuntimeError("Unknown numpy type: %s" % (self.data.dtype.type))
+        dats = self.data.tostring()
+        self.pilimage = Image.frombuffer(mode1, size, dats, "raw", mode2, 0, 1)
 
         return self.pilimage
 
@@ -191,7 +173,7 @@ class fabioimage(object):
                 coords[0:3:2] = [coords[2], coords[0]]
             if coords[1] > coords[3]:
                 coords[1:4:2] = [coords[3], coords[1]]
-            #in fabian: normally coordinates are given as (x,y) whereas
+            # in fabian: normally coordinates are given as (x,y) whereas
             # a matrix is given as row,col
             # also the (for whichever reason) the image is flipped upside
             # down wrt to the matrix hence these tranformations
@@ -202,7 +184,6 @@ class fabioimage(object):
         return (slice(int(fixme[0]), int(fixme[2]) + 1) ,
                  slice(int(fixme[1]), int(fixme[3]) + 1))
 
-
     def integrate_area(self, coords):
         """
         Sums up a region of interest
@@ -210,7 +191,7 @@ class fabioimage(object):
         if len(coords) == 2 -> use as slices
         floor -> ? removed as unused in the function.
         """
-        if self.data == None:
+        if self.data is None:
             # This should return NAN, not zero ?
             return 0
         if len(coords) == 4:
@@ -237,7 +218,7 @@ class fabioimage(object):
 
     def getstddev(self):
         """ return the standard deviation """
-        if self.stddev == None:
+        if self.stddev is None:
             self.stddev = self.data.std(dtype=numpy.double)
         return self.stddev
 
@@ -271,7 +252,7 @@ class fabioimage(object):
 
 
         """
-        if self.data == None:
+        if self.data is None:
             raise Exception('Please read in the file you wish to rebin first')
 
         if (self.dim1 % x_rebin_fact != 0) or (self.dim2 % y_rebin_fact != 0):
@@ -282,12 +263,12 @@ class fabioimage(object):
             shapeIn = self.data.shape
             shapeOut = (shapeIn[0] / y_rebin_fact, shapeIn[1] / x_rebin_fact)
             binsize = y_rebin_fact * x_rebin_fact
-            if binsize < 50: #method faster for small binning (4x4)
+            if binsize < 50:  # method faster for small binning (4x4)
                 out = numpy.zeros(shapeOut, dtype="float64")
                 for j in range(x_rebin_fact):
                     for i in range(y_rebin_fact):
                         out += dataIn[i::y_rebin_fact, j::x_rebin_fact]
-            else: #method faster for large binning (8x8)
+            else:  # method faster for large binning (8x8)
                 temp = self.data.astype("float64")
                 temp.shape = (shapeOut[0], y_rebin_fact, shapeOut[1], x_rebin_fact)
                 out = temp.sum(axis=3).sum(axis=1)
@@ -300,7 +281,7 @@ class fabioimage(object):
         self.dim1 = self.dim1 / x_rebin_fact
         self.dim2 = self.dim2 / y_rebin_fact
 
-        #update header
+        # update header
         self.update_header()
 
     def write(self, fname):
@@ -337,6 +318,9 @@ class fabioimage(object):
         by default pass in a dict of key, values.
         """
         self.header.update(kwds)
+        for key in kwds:
+            if key not in self.header_keys:
+                 self.header_keys.append(key)
 
     def read(self, filename, frame=None):
         """
@@ -372,14 +356,25 @@ class fabioimage(object):
         Return an object which can be used for "read" and "write"
         ... FIXME - what about seek ?
         """
+
+        if hasattr(fname, "read") and hasattr(fname, "write"):
+            # It is already something we can use
+            if "name" in dir(fname):
+                self.header["filename"] =self.filename= fname.name
+            else:
+                self.filename = self.header["filename"] = "stream"
+                try:
+                    setattr(fname, "name", self.filename)
+                except AttributeError:
+                    #cStringIO
+                    logger.warning("Unable to set filename attribute to stream (cStringIO?) of type %s"%type(fname))
+            return fname
+
         fileObject = None
         self.filename = fname
         self.filenumber = fabioutils.extract_filenumber(fname)
 
-        if hasattr(fname, "read") and hasattr(fname, "write"):
-            # It is already something we can use
-            return fname
-        if isinstance(fname, (str, unicode)):
+        if isinstance(fname, fabioutils.StringTypes):
             self.header["filename"] = fname
             if os.path.splitext(fname)[1] == ".gz":
                 fileObject = self._compressed_stream(fname,
@@ -418,7 +413,7 @@ class fabioimage(object):
         if self._need_a_real_file and mode[0] == "r":
             fo = python_uncompress(fname, mode)
 #            fobj = os.tmpfile()
-            #problem when not administrator under certain flavors of windows
+            # problem when not administrator under certain flavors of windows
             tmpfd, tmpfn = tempfile.mkstemp()
             os.close(tmpfd)
             fobj = fabioutils.File(tmpfn, "w+b")
@@ -427,7 +422,7 @@ class fabioimage(object):
             fobj.seek(0)
         elif self._need_a_seek_to_read and mode[0] == "r":
             fo = python_uncompress(fname, mode)
-            fobj = fabioutils.StringIO(fo.read(), fname, mode)
+            fobj = fabioutils.BytesIO(fo.read(), fname, mode)
         else:
             fobj = python_uncompress(fname, mode)
         return fobj
@@ -437,7 +432,7 @@ class fabioimage(object):
         Convert a fabioimage object into another fabioimage object (with possible conversions)
         @param dest: destination type "EDF", "edfimage" or the class itself
         """
-        if type(dest) in types.StringTypes:
+        if type(dest) in fabioutils.StringTypes:
             dest = dest.lower()
             modules = []
             for val  in fabioutils.FILETYPES.values():
@@ -473,7 +468,7 @@ class fabioimage(object):
         if klass is None:
             logger.warning("Unrecognized destination format: %s " % dest)
             return self
-        other = klass() #temporary instance (to be overwritten)
+        other = klass()  # temporary instance (to be overwritten)
         other = klass(data=converters.convert_data(self.classname, other.classname, self.data),
                     header=converters.convert_header(self.classname, other.classname, self.header))
         return other
