@@ -1,18 +1,47 @@
 #!/usr/bin/env python
-#coding: utf8
+# coding: utf-8
+from __future__ import absolute_import, print_function, with_statement, division
 
-"""
+__doc__ = """
 General purpose utilities functions for fabio
 """
-# get ready for python3
-from __future__ import with_statement, print_function
+__author__ = "Jérôme Kieffer"
+__contact__ = "Jerome.Kieffer@ESRF.eu"
+__license__ = "GPL"
+__copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
+__date__ = "08/01/2015"
+__status__ = "stable"
+__docformat__ = 'restructuredtext'
+
 import re, os, logging, threading, sys
-import StringIO as stringIO
 logger = logging.getLogger("fabioutils")
+try:
+    import six
+    if tuple(int(i) for i in six.__version__.split(".")[:2]) < (1, 8):
+        raise ImportError("Six version is too old")
+except ImportError:
+    from .third_party import six
+
+
+if six.PY2:
+    bytes = str
+    FileIO = file
+    StringTypes = (str, unicode)
+    to_str = lambda s: str(s)
+else:
+    StringTypes = (str, bytes)
+    unicode = str
+    from io import FileIO
+    to_str = lambda s: str(s, "ASCII")
+
 from .compression import bz2, gzip
 import traceback
 from math import ceil
 
+if sys.version_info < (3, 4):
+    from threading import _Semaphore as _Semaphore
+else:
+    from threading import Semaphore as _Semaphore
 
 
 FILETYPES = {
@@ -39,7 +68,7 @@ FILETYPES = {
              }
 
 # Add bzipped and gzipped
-for key in FILETYPES.keys():
+for key in list(FILETYPES.keys()):
     FILETYPES[key + ".bz2"] = FILETYPES[key]
     FILETYPES[key + ".gz"] = FILETYPES[key]
 
@@ -205,7 +234,7 @@ class FilenameObject(object):
                 stem, numstring, postnum = numstem(".".join(parts[:-1]))
                 num = int(numstring)
                 ndigit = len(numstring)
-            except Exception, err:
+            except Exception as err:
                 # There is no number - hence make num be None, not 0
                 logger.debug("l176: %s" % err)
                 num = None
@@ -226,14 +255,14 @@ class FilenameObject(object):
                     ndigit = len(parts[-1])
                     typ = ['bruker']
                     stem = ".".join(parts[:-1]) + "."
-                except Exception, err:
+                except Exception as err:
                     logger.debug("l196: %s" % err)
                     typ = None
                     extn = "." + parts[-1] + extn
                     numstring = ""
                     try:
                         stem , numstring, postnum = numstem(".".join(parts[:-1]))
-                    except Exception, err:
+                    except Exception as err:
                         logger.debug("l202: %s" % err)
                         raise
                     if numstring.isdigit():
@@ -255,18 +284,18 @@ def numstem(name):
     Match 1 or more digits going backwards from the end of the string
     """
     reg = re.compile(r"^(.*?)(-?[0-9]{0,9})(\D*)$")
-    #reg = re.compile("""(\D*)(\d\d*)(\w*)""")
+    # reg = re.compile("""(\D*)(\d\d*)(\w*)""")
     try:
         res = reg.match(name).groups()
-        #res = reg.match(name[::-1]).groups()
-        #return [ r[::-1] for r in res[::-1]]
-        if len(res[0]) == len(res[1]) == 0: # Hack for file without number
+        # res = reg.match(name[::-1]).groups()
+        # return [ r[::-1] for r in res[::-1]]
+        if len(res[0]) == len(res[1]) == 0:  # Hack for file without number
             return [res[2], '', '']
         return [ r for r in res]
-    except AttributeError: # no digits found
+    except AttributeError:  # no digits found
         return [name, "", ""]
 
-#@deprecated
+# @deprecated
 def deconstruct_filename(filename):
     """
     Function for backward compatibility.
@@ -319,7 +348,7 @@ def isAscii(name, listExcluded=None):
     """
     isascii = None
     try:
-        name.decode("ascii")
+        name.encode("ASCII")
     except UnicodeDecodeError:
         isascii = False
     else:
@@ -360,21 +389,22 @@ def nice_int(s):
         return int(float(s))
 
 
-class StringIO(stringIO.StringIO):
+class BytesIO(six.BytesIO):
     """
-    just an interface providing the name and mode property to a StringIO
+    just an interface providing the name and mode property to a BytesIO
 
     BugFix for MacOSX mainly
     """
     def __init__(self, data, fname=None, mode="r"):
-        stringIO.StringIO.__init__(self, data)
-        self.closed = False
+        six.BytesIO.__init__(self, data)
+        if not "closed" in dir(self):
+            self.closed = False
         if fname == None:
             self.name = "fabioStream"
         else:
             self.name = fname
         self.mode = mode
-        self.lock = threading.Semaphore()
+        self.lock = _Semaphore()
         self.__size = None
 
     def getSize(self):
@@ -390,7 +420,8 @@ class StringIO(stringIO.StringIO):
         self.__size = size
     size = property(getSize, setSize)
 
-class File(file):
+
+class File(FileIO):
     """
     wrapper for "file" with locking
     """
@@ -413,9 +444,13 @@ class File(file):
 
         'U' cannot be combined with 'w' or '+' mode.
         """
-        file.__init__(self, name, mode, buffering)
-        self.lock = threading.Semaphore()
+        if six.PY2:
+            FileIO.__init__(self, name, mode, buffering)
+        else:  # for python3 we drop buffering
+            FileIO.__init__(self, name, mode)
+        self.lock = _Semaphore()
         self.__size = None
+
     def getSize(self):
         if self.__size is None:
             logger.debug("Measuring size of %s" % self.name)
@@ -431,7 +466,7 @@ class File(file):
         """
         Close the file.
         """
-        return file.close(self)
+        return FileIO.close(self)
     def __enter__(self, *args, **kwargs):
         return self
     size = property(getSize, setSize)
@@ -445,6 +480,7 @@ class UnknownCompressedFile(File):
         logger.warning("No decompressor found for this type of file (are gzip anf bz2 installed ???")
         File.__init__(self, name, mode, buffering)
 
+
 if gzip is None:
     GzipFile = UnknownCompressedFile
 else:
@@ -481,14 +517,13 @@ else:
             and 9 is slowest and produces the most compression.  The default is 9.
             """
             gzip.GzipFile.__init__(self, filename, mode, compresslevel, fileobj)
-            self.lock = threading.Semaphore()
+            self.lock = _Semaphore()
             self.__size = None
-#            self.getSize()
 
         def __repr__(self):
             return "fabio." + gzip.GzipFile.__repr__(self)
 
-        def getSize(self):
+        def measure_size(self):
             if self.mode == gzip.WRITE:
                 return self.size
             if self.__size is None:
@@ -500,45 +535,7 @@ else:
                         logger.debug("Measuring size of %s: %s @ %s == %s" % (self.name, end_pos, pos, self.offset))
                         self.__size = end_pos
             return self.__size
-        if sys.version_info < (2, 7):
-            def setSize(self, value):
-                self.__size = value
-            size = property(getSize, setSize)
-            @property
-            def closed(self):
-                return self.fileobj is None
 
-            def seek(self, offset, whence=os.SEEK_SET):
-                """
-                Move to new file position.
-
-                Argument offset is a byte count.  Optional argument whence defaults to
-                0 (offset from start of file, offset should be >= 0); other values are 1
-                (move relative to current position, positive or negative), and 2 (move
-                relative to end of file, usually negative, although many platforms allow
-                seeking beyond the end of a file).  If the file is opened in text mode,
-                only offsets returned by tell() are legal.  Use of other offsets causes
-                undefined behavior.
-
-                This is a wrapper for seek to ensure compatibility with old python 2.5
-
-                Warning: Seek from end is not supported (works only for single blocks !!!)
-                This implemtents a hack
-                """
-                if whence == os.SEEK_SET:
-                    gzip.GzipFile.seek(self, offset)
-                elif whence == os.SEEK_CUR:
-                    gzip.GzipFile.seek(self, offset + self.tell())
-                elif whence == os.SEEK_END:
-                    gzip.GzipFile.seek(self, offset + self.getSize())
-
-            def __enter__(self, *args, **kwargs):
-                return self
-            def __exit__(self, *args, **kwargs):
-                """
-                Close the file.
-                """
-                return gzip.GzipFile.close(self)
 
 if bz2 is None:
     BZ2File = UnknownCompressedFile
@@ -563,7 +560,7 @@ else:
             newlines are available only when reading.
             """
             bz2.BZ2File.__init__(self, name , mode, buffering, compresslevel)
-            self.lock = threading.Semaphore()
+            self.lock = _Semaphore()
             self.__size = None
         def getSize(self):
             if self.__size is None:
@@ -586,15 +583,14 @@ else:
             return self
 
 
-
-class DebugSemaphore(threading._Semaphore):
+class DebugSemaphore(_Semaphore):
     """
     threading.Semaphore like class with helper for fighting dead-locks
     """
-    write_lock = threading._Semaphore()
+    write_lock = _Semaphore()
     blocked = []
     def __init__(self, *arg, **kwarg):
-        threading._Semaphore.__init__(self, *arg, **kwarg)
+        _Semaphore.__init__(self, *arg, **kwarg)
 
 
     def acquire(self, *arg, **kwarg):
@@ -604,7 +600,7 @@ class DebugSemaphore(threading._Semaphore):
                 sys.stderr.write(os.linesep.join(["Blocking sem %s" % id(self)] + \
                                         traceback.format_stack()[:-1] + [""]))
 
-        return threading._Semaphore.acquire(self, *arg, **kwarg)
+        return _Semaphore.acquire(self, *arg, **kwarg)
 
     def release(self, *arg, **kwarg):
         with self.write_lock:
@@ -612,7 +608,7 @@ class DebugSemaphore(threading._Semaphore):
             if uid in self.blocked:
                 self.blocked.remove(uid)
                 sys.stderr.write("Released sem %s %s" % (uid, os.linesep))
-        threading._Semaphore.release(self, *arg, **kwarg)
+        _Semaphore.release(self, *arg, **kwarg)
 
     def __enter__(self):
         self.acquire()
