@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#coding: utf8
+# coding: utf-8
 
 """
 Author: Andy Hammersley, ESRF
@@ -13,12 +13,13 @@ __authors__ = ["Jon Wright", "Jérôme Kieffer"]
 __contact__ = "Jerome.Kieffer@esrf.fr"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__version__ = "28 Oct 2013"
+__version__ = "06/01/2015"
 
 import numpy
 import sys
+import struct
 from .fabioimage import fabioimage
-if sys.version_info < (3.0):
+if sys.version < '3':
     bytes = str
 
 
@@ -32,14 +33,17 @@ class fit2dmaskimage(fabioimage):
         """
         # 1024 bytes gives 256x32 bit integers
         header = infile.read(1024)
-        for i, j in [ ("M", 0),
-                      ("A", 4),
-                      ("S", 8),
-                      ("K", 12)  ]:
-            if header[j] != i:
+        for i, j in [ (b"M", 0),
+                      (b"A", 4),
+                      (b"S", 8),
+                      (b"K", 12)  ]:
+            if header[j] != i[0]:
                 raise Exception("Not a fit2d mask file")
         fit2dhdr = numpy.fromstring(header, numpy.int32)
-        self.dim1 = fit2dhdr[4] # 1 less than Andy's fortran
+        # Enforce little endian
+        if not numpy.little_endian:
+             fit2dhdr.byteswap(True)
+        self.dim1 = fit2dhdr[4]  # 1 less than Andy's fortran
         self.dim2 = fit2dhdr[5]
 
 
@@ -63,6 +67,9 @@ class fit2dmaskimage(fabioimage):
 
         # Now to unpack it
         data = numpy.fromstring(data, numpy.uint8)
+        if not numpy.little_endian:
+             data.byteswap(True)
+
         data = numpy.reshape(data, (self.dim2, num_ints * 4))
 
         result = numpy.zeros((self.dim2, num_ints * 4 * 8), numpy.uint8)
@@ -92,30 +99,22 @@ class fit2dmaskimage(fabioimage):
         """
         Try to write a file
         """
-        header = numpy.zeros(1024, dtype=numpy.uint8)
-        header[0] = 77 #M
-        header[4] = 65 #A
-        header[8] = 83 #S
-        header[12] = 75 #K
-        header[24] = 1 #1
-        str1 = numpy.array([self.dim1], numpy.uint32).tostring()
-        str2 = numpy.array([self.dim2], numpy.uint32).tostring()
-        if not numpy.little_endian:
-            str1 = str1[-1::-1]
-            str2 = str2[-1::-1]
-        for i, c in zip(range(16, 20), str1):
-            header[i] = ord(c)
-        for i, c in zip(range(20, 24), str2):
-            header[i] = ord(c)
+        header = bytearray(b"\x00"*1024)
+        header[0] = 77  # M
+        header[4] = 65  # A
+        header[8] = 83  # S
+        header[12] = 75  # K
+        header[24] = 1  # 1
+        header[16:20] = struct.pack("<I", self.dim1)
+        header[20:24] = struct.pack("<I", self.dim2)
         compact_array = numpy.zeros((self.dim2, ((self.dim1 + 31) // 32) * 4), dtype=numpy.uint8)
         large_array = numpy.zeros((self.dim2, ((self.dim1 + 31) // 32) * 32), dtype=numpy.uint8)
         large_array[:self.dim2, :self.dim1] = (self.data != 0)
-        order = 1
         for i in range(8):
+            order = (1 << i)
             compact_array += large_array[:, i::8] * order
-            order *= 2
         with self._open(fname, mode="wb") as outfile:
-            outfile.write(header)
+            outfile.write(bytes(header))
             outfile.write(compact_array.tostring())
 
     @staticmethod
