@@ -5,7 +5,7 @@ Read the fit2d ascii image output
         + Jon Wright, ESRF
 """
 # Get ready for python3:
-from __future__ import with_statement, print_function
+from __future__ import absolute_import, print_function, with_statement, division
 import numpy
 
 from .fabioimage import fabioimage
@@ -66,7 +66,7 @@ class fit2dspreadsheetimage(fabioimage):
             assert self.data.shape == (self.dim2, self.dim1)
 
         except:
-            raise IOError, "Error reading ascii"
+            raise IOError("Error reading ascii")
 
         self.resetvals()
         # ensure the PIL image is reset
