@@ -12,17 +12,17 @@ Authors:
 * Jon Wright:
   European Synchrotron Radiation Facility;
   Grenoble (France)
-         
+
 marccdimage can read MarCCD and MarMosaic images including header info.
 
 JPW : Use a parser in case of typos (sorry?)
 
 """
 # Get ready for python3:
-from __future__ import with_statement, print_function
+from __future__ import with_statement, print_function, absolute_import
 
-# Base this on the tifimage (as marccd seems to be tiff with a 
-# special header 
+# Base this on the tifimage (as marccd seems to be tiff with a
+# special header
 
 from .tifimage import tifimage
 import logging
@@ -34,18 +34,18 @@ logger = logging.getLogger("marccdimage")
 CDEFINITION = """
 typedef struct frame_header_type {
          /* File/header format parameters (256 bytes) */
-         UINT32        header_type;  /* flag for header type  
+         UINT32        header_type;  /* flag for header type
                                            (can be  used as magic number) */
          char header_name[16];           /* header name (MMX) */
          UINT32        header_major_version;   /* header_major_version  (n.) */
          UINT32        header_minor_version;   /* header_minor_version  (.n) */
-         UINT32        header_byte_order;/* BIG_ENDIAN (Motorola,MIPS);  
+         UINT32        header_byte_order;/* BIG_ENDIAN (Motorola,MIPS);
                                             LITTLE_ENDIAN (DEC, Intel) */
-         UINT32        data_byte_order;  /* BIG_ENDIAN (Motorola,MIPS);  
+         UINT32        data_byte_order;  /* BIG_ENDIAN (Motorola,MIPS);
                                             LITTLE_ENDIAN (DEC, Intel) */
          UINT32        header_size;      /* in bytes                     */
          UINT32        frame_type;       /* flag for frame type */
-         UINT32        magic_number;     /* to be used as a flag - 
+         UINT32        magic_number;     /* to be used as a flag -
                                             usually  to indicate new file */
          UINT32        compression_type; /* type of image compression    */
          UINT32        compression1;     /* compression parameter 1 */
@@ -58,13 +58,13 @@ typedef struct frame_header_type {
          UINT32        nfast;            /* number of pixels in one line */
          UINT32        nslow;            /* number of lines in image     */
          UINT32        depth;            /* number of bytes per pixel    */
-         UINT32        record_length;    /* number of pixels between 
+         UINT32        record_length;    /* number of pixels between
                                             succesive rows */
          UINT32        signif_bits;      /* true depth of data, in bits  */
          UINT32        data_type;        /* (signed,unsigned,float...) */
          UINT32        saturated_value;  /* value marks pixel as saturated */
          UINT32        sequence;         /* TRUE or FALSE */
-         UINT32        nimages;          /* total number of images - size of 
+         UINT32        nimages;          /* total number of images - size of
                                             each is nfast*(nslow/nimages) */
          UINT32        origin;           /* corner of origin             */
          UINT32        orientation;      /* direction of fast axis       */
@@ -75,17 +75,17 @@ typedef struct frame_header_type {
          UINT32        multiplexed;      /* multiplex flag */
          UINT32        nfastimages;      /* # of images in fast direction */
          UINT32        nslowimages;      /* # of images in slow direction */
-         UINT32        background_applied;/* flags correction has been applied 
+         UINT32        background_applied;/* flags correction has been applied
                                               hold magic number ? */
          UINT32        bias_applied;     /* flags correction has been applied
                                              hold magic number ? */
          UINT32        flatfield_applied;/* flags correction has been applied -
                                               hold magic number ? */
-         UINT32        distortion_applied;/*flags correction has been applied - 
+         UINT32        distortion_applied;/*flags correction has been applied -
                                               hold magic number ? */
-         UINT32        original_header_type;    /* Header/frame type from  file 
+         UINT32        original_header_type;    /* Header/frame type from  file
                                                     that frame is read from */
-         UINT32        file_saved;         /* Flag that file has been  saved, 
+         UINT32        file_saved;         /* Flag that file has been  saved,
                                               should be zeroed if modified */
          char reserve1[(64-40)*sizeof(INT32)-16];
 
@@ -145,11 +145,11 @@ typedef struct frame_header_type {
          INT32 mean_bias;                        /* 1000*mean bias value */
          INT32 photons_per_100adu;       /* photons / 100 ADUs */
          INT32 measured_bias[MAXIMAGES];/* 1000*mean bias value for each image*/
-         INT32 measured_temperature[MAXIMAGES];  /* Temperature of each  
+         INT32 measured_temperature[MAXIMAGES];  /* Temperature of each
                                                     detector in milliKelvins */
-         INT32 measured_pressure[MAXIMAGES]; /* Pressure of each  chamber 
+         INT32 measured_pressure[MAXIMAGES]; /* Pressure of each  chamber
                                                in microTorr */
-         /* Retired reserve4 when MAXIMAGES set to 9 from 16 and 
+         /* Retired reserve4 when MAXIMAGES set to 9 from 16 and
             two fields removed, and temp and pressure added
           char reserve4[(32-(5+3*MAXIMAGES))*sizeof(INT32)]
          */
@@ -176,7 +176,7 @@ typedef struct frame_header_type {
          INT32 optics_dispersion;        /* Optics param. - (*10E6) */
          INT32 optics_crossfire_x;       /* Optics param. - (microRadians) */
          INT32 optics_crossfire_y;       /* Optics param. - (microRadians) */
-         INT32 optics_angle;             /* Optics param. - (monoch.  
+         INT32 optics_angle;             /* Optics param. - (monoch.
                                                     2theta - microradians) */
          INT32 optics_polarization_x;    /* () */
          INT32 optics_polarization_y;    /* () */
@@ -197,7 +197,7 @@ typedef struct frame_header_type {
          /* Dataset parameters (512 bytes) */
          char dataset_comments[512];     /* comments, used as desired   */
          /* pad out to  3072 bytes */
-         char pad[3072-(256+128+256+(3*128)+1024+512)];     
+         char pad[3072-(256+128+256+(3*128)+1024+512)];
 
          } frame_header;
 """
@@ -226,7 +226,7 @@ MAXIMAGES = 9
 
 def make_format(c_def_string):
     """
-    Reads the header definition in c and makes the format 
+    Reads the header definition in c and makes the format
     string to pass to struct.unpack
     """
     lines = c_def_string.split("\n")
@@ -250,7 +250,7 @@ def make_format(c_def_string):
                 num = num.replace("MAXIMAGES", str(MAXIMAGES))
                 num = num.replace("sizeof(INT32)", "4")
                 times = eval(num)
-            except Exception, error:
+            except Exception as error:
                 logger.error("%s Please decode %s", error, decl)
                 raise error
         else:
@@ -274,7 +274,7 @@ def interpret_header(header, fmt, names):
     hdr = {}
     i = 0
     for name in names:
-        if hdr.has_key(name):
+        if name in hdr:
             if type(values[i]) == type("string"):
                 hdr[name] = hdr[name] + values[i]
             else:
@@ -290,7 +290,7 @@ def interpret_header(header, fmt, names):
 
 
 class marccdimage(tifimage):
-    """ Read in data in mar ccd format, also 
+    """ Read in data in mar ccd format, also
         MarMosaic images, including header info """
 
 
