@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-#-*- coding: utf8 -*-
+#-*- coding: utf-8 -*-
 """
-FabIO class for dealing with TIFF images. 
+FabIO class for dealing with TIFF images.
 In facts wraps TiffIO from V. Armando Solé (available in PyMca) or falls back to PIL
 
 Authors:
@@ -15,14 +15,14 @@ Authors:
 * Jérôme Kieffer:
   European Synchrotron Radiation Facility;
   Grenoble (France)
-   
-License: GPLv3+        
+
+License: GPLv3+
 """
 # Get ready for python3:
 from __future__ import with_statement, print_function, division
 
 __authors__ = ["Jérôme Kieffer", "Henning O. Sorensen", "Erik Knudsen"]
-__date__ = "11/07/2011"
+__date__ = "23/03/2011"
 __license__ = "GPLv3+"
 __copyright__ = "ESRF, Grenoble & Risoe National Laboratory"
 __status__ = "stable"
@@ -35,7 +35,15 @@ except ImportError:
     Image = None
 import numpy
 from .fabioimage import fabioimage
-from .TiffIO     import TiffIO
+
+try:
+    from PyMca.TiffIO import TiffIO
+except ImportError:
+    try:
+        from PyMca5.PyMca.TiffIO import TiffIO
+    except ImportError:
+        from .third_party.TiffIO import TiffIO
+
 
 PIL_TO_NUMPY = { "I;16": numpy.uint16,
                    "F": numpy.float32,
@@ -129,7 +137,7 @@ class tifimage(fabioimage):
                 #No support for now of multi-frame tiff images
                 self.data = tiffIO.getImage(0)
                 self.header = tiffIO.getInfo(0)
-        except Exception, error:
+        except Exception as error:
             logger.warning("Unable to read %s with TiffIO due to %s, trying PIL" % (fname, error))
         else:
             if self.data.ndim == 2:
@@ -179,7 +187,7 @@ class tifimage(fabioimage):
 
 
 
-#define a couple of helper classes here:         
+#define a couple of helper classes here:
 class Tiff_header(object):
     def __init__(self, string):
         if string[:4] == "II\x2a\x00":
@@ -224,7 +232,7 @@ class Image_File_Directory(object):
             for e in self.entries:
                 if (e.val == None):
                     e.extract_data(instring)
-        #do we have some more ifds in this file 
+        #do we have some more ifds in this file
         offset_next = struct.unpack_from("L", instring[offset + 2 + self.count * 12:])[0]
         return offset_next
 
